-- Copyright © 2018-2020 Nicolas Mailhot <nim@fedoraproject.org>
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <https://www.gnu.org/licenses/>.
--
-- SPDX-License-Identifier: GPL-3.0-or-later

-- Convenience lua functions used to create rpm font packages

local fedora = require "fedora.common"
local   auto = require "fedora.auto"

-- Return a normalized name
local function norm(name)
  local r = name
  r = string.gsub(r, "[%p%s]+", "-")
  r = string.gsub(r, "^-", "")
  r = string.gsub(r, "-$", "")
  r = string.lower(r)
  return r
end

-- loop over suffixlist and return name minus the first suffix that matches
-- - is used as suffix separator
-- name should have passed through norm at one point in the past
local function drop_suffix(name,suffixlist)
  local r = name
  for _, s in ipairs(suffixlist) do
    r, n = string.gsub(r, "-" .. norm(s) .. "$", "")
    if (n == 1) then break end
  end
  return r
end

-- Compute a font family name that can be used in packaging, lowercasing, using
-- - as separator, and applying  “WPF font selection model” whitepaper
-- simplifications
local function rpm_name(name)
  local r = norm(name)
  -- Normal & co
  r = drop_suffix(r,{
        "normal","book","regular","upright"})
  -- Slant
  r = drop_suffix(r,{
        "italic","ita","ital","cursive","kursiv",
        "oblique","inclined","backslanted","backslant","slanted"})
  -- Width / Stretch
  r = drop_suffix(r,{
        "ultracondensed","extra-compressed","ext-compressed","ultra-compressed","ultra-condensed",
        "extracondensed","compressed","extra-condensed","ext-condensed","extra-cond",
        "semicondensed","narrow","semi-condens",
        "semiexpanded","wide","semi-expanded","semi-extended",
        "extraexpanded","extra-expanded","ext-expanded","extra-extended","ext-extended",
        "ultraexpanded","ultra-expanded","ultra-extended",
        "condensed","cond",
        "expanded","extended"})
  -- Weight (no abbreviated suffix handling, too dangerous)
  r = drop_suffix(r,{
        "thin","extra-thin","ext-thin","ultra-thin",
        "extralight","extra-light","ext-light","ultra-light",
        "demibold","semi-bold","demi-bold",
        "extrabold","extra-bold","ext-bold","ultra-bold",
        "extrablack","extra-black","ext-black","ultra-black",
        "bold","thin","light","medium",
        "black","heavy","nord",
        "demi","ultra"})
  local tokens = {}
  for _, t in ipairs({"font","fonts"}) do
    tokens[t] = true
  end
  local ts = string.gmatch(r, "[^%-]+")
  r = ""
  for t in ts do
     if not tokens[t] then
       r = r .. "-" .. t
       tokens[t] = true
     end
  end
  r = string.gsub(r, "^-", "") .. "-fonts"
  return r
end

local namespace = "fonts"

local rads = {
  ["key"]         = {"family"},
  -- source → fonts
  ["source_from"] = {"epoch", "version", "release", "post_release", "license",
                     "url"},
  -- fonts → source
  ["source"]      = {"name", "summary", "epoch", "version", "release",
                     "post_release", "license", "url", "description"},
  -- from defaults
  ["default"]     = {"foundry", "license", "url",
                     "licenses", "licenses_exclude",
                     "docs", "docs_exclude",
                     "base_dir", "contact", "organization", "extract_dir"},
  -- computed
  ["computed"]    = {"name", "human_name", "dir", "list"},
  -- read and inherited
  ["read"]        = {"summary", "epoch", "version", "release", "post_release",
                     "description"},
  -- read but not inherited
  ["read_once"]   = {"include", "tags",
                     "fonts", "fonts_exclude",
                     "confs", "confs_exclude",
                     "experimental_confs", "experimental_confs_exclude",
                     "appstreams", "appstreams_exclude"}
}

local function suffixes(verbose)
  return fedora.all_suffixes(fedora.qualify(rads["key"], namespace), verbose)
end

local function name(suffix)
  return fedora.read(namespace .. "_name" .. suffix)
end

local function names(verbose)
  return auto.names(name, suffixes(verbose))
end

local function floop(f, index, otherargs, verbose)
  auto.pkg_floop(f, index, otherargs, name, suffixes(verbose), verbose)
end

local function radicals()
  local R = {}
  for k, v in pairs(rads) do
    R[k] = v
  end
  R["read"] = fedora.mergelists({R["read"], R["default"]})
  local all = {}
  for k, v in pairs(R) do
    all = fedora.mergelists({all, v})
  end
  R["all"] = all
  return R
end

local function init(suffix, verbose, informative)
 auto.id(namespace .. '.init("' .. suffix .. '")')
  local ismain = (suffix == "") or (suffix == "0")
  local myrads = radicals()
  fedora.inherit(myrads["read"], namespace, suffix, "include", verbose)
  fedora.mset(myrads["all"], namespace, suffix, {
    fedora.mread(myrads["default"], "default_" .. namespace, ""),
    fedora.mread(myrads["source_from"], "source", ""),
    { ["human_name"] = "%{?" .. namespace .. "_foundry" .. suffix .. ":"   ..
                       "%{"  .. namespace .. "_foundry" .. suffix .. "} }" ..
                       "%{"  .. namespace .. "_family"  .. suffix .. "}",
      ["dir"]  = "%{" .. namespace .. "_base_dir" .. suffix .. "}/" ..
                 "%{" .. namespace .. "_name"  .. suffix .. "}",
      ["list"] = "%{" .. namespace .. "_work_dir}/" ..
                 "%{" .. namespace .. "_name" .. suffix .. "}.lst"
    }}, verbose, false)
  fedora.safeset(namespace .. "_name" .. suffix,
                 rpm_name(fedora.read(namespace .. "_human_name" .. suffix)),
                 verbose)
  if informative then
    fedora.echo("Variables read or set by %%fonts_init")
    fedora.echovars(fedora.qualify(myrads["all"], namespace), suffix)
  end
end

local function env(suffix, verbose)
  local myrads = radicals(suffix, verbose)
  fedora.set_current(fedora.qualify(myrads["all"], namespace), suffix, verbose)
  fedora.set_verbose(verbose)
end

local function pkg(suffix, verbose)
 auto.id(namespace .. '.pkg("' .. suffix .. '")')
  env( suffix, verbose)
  local myrads = radicals(suffix, verbose)
  fedora.alias(myrads["source"], namespace, suffix, "source",  "", verbose)
  print(rpm.expand([[
%__]] .. namespace .. [[_pkg
]]))
end

return {
  rpm_name = rpm_name,
  suffixes = suffixes,
  names    = names,
  floop    = floop,
  init     = init,
  env      = env,
  pkg      = pkg,
}
