-- Copyright © 2018-2020 Nicolas Mailhot <nim@fedoraproject.org>
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <https://www.gnu.org/licenses/>.
--
-- SPDX-License-Identifier: GPL-3.0-or-later

-- Convenience lua functions used to create rpm font meta packages

local fedora = require "fedora.common"
local   auto = require "fedora.auto"
local  fonts = require "fedora.srpm.fonts"

local namespace = "fonts_meta"

local rads = {
  ["key"]      = {"packages", "all"},
  ["read"]     = {"epoch", "version", "release", "post_release", "license",
                  "url"},
  ["computed"] = {"name", "summary", "tags", "description"}
}

local function suffixes()
  return fedora.all_suffixes(fedora.qualify(rads["key"], namespace))
end

local function floop(f, index, otherargs, verbose)
  local        range = suffixes(verbose)
  auto.floop(f, index, range, otherargs)
end

local function radicals()
  local R = {}
  for k, v in pairs(rads) do
    R[k] = v
  end
  local all = {}
  for k, v in pairs(R) do
    all = fedora.mergelists({all, v})
  end
  R["all"] = all
  return R
end

local function init(suffix, verbose, informative)
 auto.id(namespace .. '.init("' .. suffix .. '")')
  local ismain = (suffix == "") or (suffix == "0")
  local myrads = radicals()
  local tags = fedora.expand("%{?" .. namespace .. "_tags" .. suffix .. "}")
  if fedora.readbool(namespace .. "_all" .. suffix) then
    for _, fs in ipairs(fonts.suffixes(verbose)) do
      tags = tags .. "\nRequires(meta): %{fonts_name" .. fs .. "}"
    end
  else
    fsl = fedora.read(namespace .. "_packages" .. suffix)
    if fsl then
      for fs in fsl:gmatch("[^%s%p]+") do
        tags = tags .. "\nRequires(meta): %{fonts_name" .. fs .. "}"
      end
    end
  end
  fedora.mset(myrads["all"], namespace, suffix, {{
      ["name"]        = [[%{lua:
                          local fedora = require "fedora.common"
                          if fedora.read("source_name") then
                            print(
                              fedora.read("source_name"):gsub("-fonts$", "") ..
                              "-fonts-all")
                          end
                        }]],
      ["summary"]     = "All the font packages, created in %{source_name}",
      ["tags"]        = tags,
      ["description"] = [[
%{?source_description}

This meta-package installs all the font packages, created in the
%{source_name} source package.

It is provided as end-user convenience. Do not depend on it in other packages.
]],
    }}, verbose, false)
  if informative then
    fedora.echo("Variables read or set by %%fonts_meta_init")
    fedora.echovars(fedora.qualify(myrads["all"], namespace), suffix)
  end
end

local function env(suffix, verbose)
  local myrads = radicals(suffix, verbose)
  fedora.set_current(fedora.qualify(myrads["all"], namespace), suffix, verbose)
end

local function pkg(suffix, verbose)
 auto.id(namespace .. '.pkg("' .. suffix .. '")')
  env( suffix, verbose)
  print(rpm.expand([[
%__]] .. namespace .. [[_pkg
]]))
end

return {
  suffixes = suffixes,
  floop    = floop,
  init     = init,
  env      = env,
  pkg      = pkg,
}
